import type { Writer } from 'ckeditor5/src/engine';

/**
 * Creates a `<accordionItem>` model element with the necessary child elements.
 *
 * @param writer
 *   The model writer.
 * @returns
 *   The accordion item model element.
 */
export function createAccordionItem(writer: Writer) {
  const accordionItem = writer.createElement('bootstrapAccordionItem');
  const accordionHeader = writer.createElement('bootstrapAccordionHeader');
  const accordionButton = writer.createElement('bootstrapAccordionButton');
  const accordionCollapse = writer.createElement('bootstrapAccordionCollapse');
  const accordionBody = writer.createElement('bootstrapAccordionBody');

  writer.append(accordionHeader, accordionItem);
  writer.append(accordionButton, accordionHeader);
  writer.append(accordionCollapse, accordionItem);
  writer.append(accordionBody, accordionCollapse);

  // The accordionBody text content will automatically be wrapped in a
  // `<p>`.
  writer.appendElement('paragraph', accordionBody);

  return { accordionItem, accordionHeader, accordionButton, accordionCollapse, accordionBody };
}
