<?php

namespace Drupal\ckeditor5_bootstrap_accordion\Plugin\Filter;

use Drupal\Component\Utility\Html;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 * Defines a filter used by the Bootstrap Accordion plugin.
 *
 * @Filter(
 *   id = "filter_bootstrap_accordion",
 *   title = @Translation("Accordion enabler"),
 *   description = @Translation("Enables accordions generated by the CKEditor 5 Bootstrap Accordion plugin to work correctly by adding the attributes required by Bootstrap."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
 *   weight = 10
 * )
 */
class BootstrapAccordion extends FilterBase {

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $result = new FilterProcessResult($text);

    if (stristr($text, ' data-accordion-id') === FALSE) {
      return $result;
    }

    $dom = Html::load($text);
    $xpath = new \DOMXPath($dom);

    foreach ($xpath->query("//div[contains(concat(' ', @class, ' '), ' accordion ')]") as $accordion) {
      $accordionIdFromAttribute = $accordion->getAttribute('data-accordion-id');
      $accordionId = 'accordion-' . $accordionIdFromAttribute;
      $accordion->setAttribute('id', $accordionId);
      $accordionItemCount = 1;
      foreach ($xpath->query("//div[@id='" . $accordionId . "']//div[contains(concat(' ', @class, ' '), ' accordion-item ')]") as $accordionItem) {
        $accordionClasses = preg_split('/\s+/', $accordion->getAttribute('class'));
        $accordionItemsStayOpen = in_array('accordion-items-stay-open', $accordionClasses);
        $accordionCollapseId = $accordionId . '-' . ($accordionItemCount++);
        foreach ($accordionItem->childNodes as $childItem) {
          if ($childItem->hasAttribute('class')) {
            if (in_array('accordion-header', preg_split('/\s+/', $childItem->getAttribute('class')))) {
              foreach ($childItem->childNodes as $accordionButton) {
                if ($accordionButton->hasAttribute('class') && ($accordionButtonClasses = preg_split('/\s+/', $accordionButton->getAttribute('class'))) && in_array('accordion-button', $accordionButtonClasses)) {
                  // Adds the necessary attributes to the accordion-button.
                  $accordionButton->setAttribute('role', 'button');
                  $accordionButton->setAttribute('data-bs-toggle', 'collapse');
                  $accordionButton->setAttribute('data-bs-target', '#' . $accordionCollapseId);
                  $accordionButton->setAttribute('aria-expanded', in_array('collapsed', $accordionButtonClasses) ? 'false' : 'true');
                  $accordionButton->setAttribute('aria-controls', $accordionCollapseId);
                  if ($accordionButton->hasAttribute('href')) {
                    $accordionButton->setAttribute('href', '#' . $accordionCollapseId);
                  }
                }
              }
            }
            elseif (in_array('accordion-collapse', preg_split('/\s+/', $childItem->getAttribute('class')))) {
              // Adds the necessary attributes to the accordion-collapse.
              $childItem->setAttribute('id', $accordionCollapseId);
              if (!$accordionItemsStayOpen) {
                $childItem->setAttribute('data-bs-parent', '#' . $accordionId);
              }
            }
          }
        }
      }
    }

    $result->setProcessedText(Html::serialize($dom));

    return $result;
  }

}
